'use strict';

var branchToDeploy;
var chalk = require('chalk');
var defaultBranch = 'develop';
var foundProc;
var Menu = require('terminal-menu');
var menu;
var pathToJson;
var rebranchDependencies = require('rebranch-dependencies');
var spawn = require('child_process').spawn;

module.exports = function drawMenu(options) {
    var pkgJson = options.json;

    pathToJson = options.pathToJson;

    menu = Menu({ /* jshint ignore: line */
        width: 50,
        x: 3,
        y: 2
    });

    branchToDeploy = defaultBranch;

    menu.reset();
    menu.write('DEPLOY YOUR APP FLAWLESSLY\n');

    menu.write('-------------------------\n');

    menu.write('Selected app:\n');
    menu.write(pkgJson.name + '@' + pkgJson.version + '\n');

    menu.write('-------------------------\n');

    menu.write('Select your environment:\n\n');
    menu.add('X DEVELOPMENT', changeEnv.bind(this, defaultBranch));
    menu.add('- STAGING', changeEnv.bind(this, 'staging'));
    menu.add('- PRODUCTION', changeEnv.bind(this, 'master'));

    menu.write('-------------------------\n');

    menu.write('Executing the DEPLOY menu point, will process\n');
    menu.write('these things:\n');
    menu.write(' 1) change branch of to-install deployment packages\n');
    menu.write(' 2) install npm packages\n');
    menu.write(' 3) restart pm2\n\n');

    menu.add('DEPLOY', deploy);

    menu.write('-------------------------\n');

    menu.add('EXIT', exit);

    menu.on('close', function closeMenu() {
        process.stdin.setRawMode(false);
        process.stdin.end();
    });

    process.stdin.pipe(menu.createStream()).pipe(process.stdout);
    process.stdin.setRawMode(true);

    function exit() {
        menu.close();
    }

    function deploy() {
        menu.close();

        rebranch();
    }

    function rebranch() {
        console.log('Rebranching dependencies ...');
        rebranchDependencies.rebranchAndWritePackageJson(pathToJson,
                branchToDeploy);
        console.log(chalk.green('Successfully rebranched.'));

        gitPull();
    }

    function gitPull() {
        var pull;

        console.log(chalk.bold('Starting git pull ...'));

        pull = spawn('git', ['pull']);

        pull.stdout.on('data', function (data) {
            console.log(chalk.cyan(data));
        });

        pull.stderr.on('data', function (data) {
            console.error(chalk.yellow(data));
        });

        pull.on('close', function gitPullDone(code) {
            if (code !== 0) {
                var msg = chalk.bold.red('git pull exited with code: ' + code);
                console.error(msg);
                throw new Error(msg);
            }

            console.log(chalk.green.bold('git pull done.'));

            npmInstall();
        });

    }

    function npmInstall() {
        var npmI;

        console.log(chalk.bold('Starting npm i ...'));

        npmI = spawn('npm', ['i']);

        npmI.stdout.on('data', function (data) {
            console.log(chalk.green(data));
        });

        npmI.stderr.on('data', function (data) {
            console.error(chalk.yellow(data));
        });

        npmI.on('close', function gitPullDone(code) {
            if (code !== 0) {
                var msg = chalk.bold.red('npm i exited with code: ' + code);
                console.error(msg);
                throw new Error(msg);
            }

            console.log(chalk.green.bold('npm i done.'));

            findProcessInPm2();
        });
    }

    function findProcessInPm2() {
        var json,
            jsonString,
            pm2List;

        jsonString = '';

        console.log(chalk.bold('Will now find pm2 process ...'));

        pm2List = spawn('pm2', ['jlist']);

        pm2List.stdout.on('data', function (buffer) {
            jsonString += buffer.toString();
        });

        pm2List.stderr.on('data', function (data) {
            console.error(chalk.yellow(data));
        });

        pm2List.on('close', function pm2ListOut(code) {
            var found,
            msg;

            if (code !== 0) {
                msg = chalk.bold.red('pm2 jlist exited with code: ' + code);
                console.error(msg);
                throw new Error(msg);
            }

            console.log(chalk.green.bold('Parsing pm2 jlist ...'));

            json = JSON.parse(jsonString);

            // Try to find process, where this cwd belongs to.
            found = json.some(function(_proc) {
                var _found;

                foundProc = {
                    cwd: _proc.pm2_env.pm_cwd,
                    execDir: _proc.pm2_env.pm_exec_path,
                    name: _proc.name,
                    pid: _proc.pid,
                    pm2Id: _proc.pm_id
                };

                _found = foundProc.cwd.indexOf(process.cwd()) > -1 ||
                    foundProc.execDir.indexOf(process.cwd()) > -1;

                return _found;
            });

            if (!found) {
                msg = chalk.red.bold('Could not find process in pm2. ' +
                            'Exiting now.');
                console.error(msg);
                process.exit(1);
            }

            console.log(chalk.green.bold('Found process in pm2.'));
            console.log(chalk.cyan(foundProc));

            pm2Restart();

            return;

        });
    }

    function pm2Restart() {
        var pm2Rest;

        console.log(chalk.bold('Restarting pm2 process ...'));

        pm2Rest = spawn('pm2', ['restart', foundProc.pm2Id]);

        pm2Rest.stdout.on('data', function (data) {
            console.log(chalk.green(data));
        });

        pm2Rest.stderr.on('data', function (data) {
            console.error(chalk.yellow(data));
        });

        pm2Rest.on('close', function gitPullDone(code) {
            if (code !== 0) {
                var msg = chalk.bold.red('npm i exited with code: ' + code);
                console.error(msg);
                throw new Error(msg);
            }

            console.log(chalk.green.bold('Restarted pm2 process.'));

            console.log(chalk.green.bold('Deployment successful.'));

        });
    }

    function resetSelectedEnv() {
        menu.items.forEach(function(item, index) {
            item.label = item.label.replace(/^X/, '-');
            menu.items[index] = item;
        });
    }

    // Reset background to blue, to prevent some failures in displaying
    // colors correctly.
    // Chalk will reset to 49m, but this colourful terminal is interpreting
    // these reset faulty, because it uses other colours than in default tty
    // prompt.
    function resetColor(label) {
        label = label + '\u001b[44m';
        return label;
    }

    function changeEnv(_branch, label, index) {
        var item = menu.items[index];
        branchToDeploy = _branch;

        resetSelectedEnv();
        // Chalk will not func., because the terminal module calculates
        // to-draw size from label-length for calc. background fillment.
        //item.label = item.label.replace(/^.*\)/, chalk.bold('X)'));
        item.label = item.label.replace(/^[X\-]/, 'X');
        resetColor(item.label);
        menu.items[index] = item;
        menu._draw();
    }
};

